@@ -1585,6 +1585,32 @@ body(SMFICTX *ctx, u_char *text, size_t len)
     return SMFIS_CONTINUE;
 }
 
+/**********************************************************************
+*%FUNCTION: remove_carriage_return
+*%ARGUMENTS:
+* str -- pointer to string to modify
+*%DESCRIPTION:
+* removes carriage returns from a string, header values should not
+* contain "\r" chars
+***********************************************************************/
+static void remove_carriage_return(char *str)
+{
+    char t = '\r';
+    int i,j;
+    i = 0;
+    while(i < strlen(str))
+    {
+        if (str[i] == t)
+        {
+            for (j = i; j < strlen(str); j++) {
+                str[j] = str[j+1];
+	    }
+        } else {
+	    i++;
+	}
+    }
+}
+
 /**********************************************************************
 *%FUNCTION: eom
 *%ARGUMENTS:
@@ -1617,12 +1643,20 @@ eom(SMFICTX *ctx)
 
     struct stat statbuf;
     dynamic_buffer dbuf;
-    struct timeval start, finish;
+    struct timespec start, finish, diff;
     int rejecting;
 
     DEBUG_ENTER("eom");
     if (LogTimes) {
-	gettimeofday(&start, NULL);
+#ifdef HAVE_CLOCK_MONOTONIC
+	if(clock_gettime(CLOCK_MONOTONIC, &start) == -1) {
+	  syslog(LOG_INFO, "%s: Error in clock_gettime", data->qid);
+	}
+#else
+	if(clock_gettime(CLOCK_REALTIME, &start) == -1) {
+	  syslog(LOG_INFO, "%s: Error in clock_gettime", data->qid);
+	}
+#endif
     }
 
     /* Close output file */
@@ -1879,6 +1913,7 @@ eom(SMFICTX *ctx)
 		if (hdr && val) {
 		    percent_decode(hdr);
 		    percent_decode(val);
+		    remove_carriage_return(val);
 		    MD_SMFI_TRY(smfi_addheader, (ctx, hdr, val));
 		}
 	    }
@@ -1892,6 +1927,7 @@ eom(SMFICTX *ctx)
 		    percent_decode(hdr);
 		    percent_decode(count);
 		    percent_decode(val);
+		    remove_carriage_return(val);
 		    if (sscanf(count, "%d", &j) != 1 || j < 0) {
 			j = 0; /* 0 means add header at the top */
 		    }
@@ -1915,6 +1951,7 @@ eom(SMFICTX *ctx)
 		    percent_decode(hdr);
 		    percent_decode(count);
 		    percent_decode(val);
+		    remove_carriage_return(val);
 		    if (sscanf(count, "%d", &j) != 1 || j < 1) {
 			j = 1;
 		    }
@@ -2018,21 +2055,27 @@ eom(SMFICTX *ctx)
     if (rbuf != result) free(rbuf);
 
     if (LogTimes) {
-	long sec_diff, usec_diff;
+	long msec_diff;
 
-	gettimeofday(&finish, NULL);
-
-	sec_diff = finish.tv_sec - start.tv_sec;
-	usec_diff = finish.tv_usec - start.tv_usec;
-
-	if (usec_diff < 0) {
-	    usec_diff += 1000000;
-	    sec_diff--;
+#ifdef HAVE_CLOCK_MONOTONIC
+	if(clock_gettime(CLOCK_MONOTONIC, &finish) != -1) {
+	  timespecsub(&finish, &start, &diff);
+	  // convert to milliseconds
+          msec_diff = diff.tv_nsec / 1000000;
+	  syslog(LOG_INFO, "%s: Filter time is %ldms", data->qid, msec_diff);
+	} else {
+	  syslog(LOG_INFO, "%s: Filter time cannot be calculated", data->qid);
 	}
-
-	/* Convert to milliseconds */
-	sec_diff = sec_diff * 1000 + (usec_diff / 1000);
-	syslog(LOG_INFO, "%s: Filter time is %ldms", data->qid, sec_diff);
+#else
+	if(clock_gettime(CLOCK_REALTIME, &finish) != -1) {
+	  timespecsub(&finish, &start, &diff);
+	  // convert to milliseconds
+          msec_diff = diff.tv_nsec / 1000000;
+	  syslog(LOG_INFO, "%s: Filter time is %ldms", data->qid, msec_diff);
+	} else {
+	  syslog(LOG_INFO, "%s: Filter time cannot be calculated", data->qid);
+	}
+#endif
     }
 
     DEBUG(syslog(LOG_DEBUG, "%p: %s(%d): EXIT %s: %d", ctx, __FILE__, __LINE__, "eom", r));
@@ -2267,6 +2310,7 @@ main(int argc, char **argv)
     int nodaemon = 0;
     char buf[SMALLBUF];
     int got_p_option = 0;
+    char *sockfile = NULL;
     int kidpipe[2];
     char kidmsg[256];
     int pidfile_fd = -1;
@@ -2416,7 +2460,8 @@ main(int argc, char **argv)
 	    break;
 	case 'X':
 	    if (scan_body) {
-		free(scan_body);
+                fprintf(stderr, "%s: Cannot use multiple '-X' options\n", argv[0]);
+                exit(EXIT_FAILURE);
 	    }
 	    scan_body = strdup("");
 	    if (!scan_body) {
@@ -2427,7 +2472,8 @@ main(int argc, char **argv)
 
 	case 'L':
 	    if (EquivToLoopback) {
-		free(EquivToLoopback);
+                fprintf(stderr, "%s: Cannot use multiple '-L' options\n", argv[0]);
+                exit(EXIT_FAILURE);
 	    }
 	    EquivToLoopback = strdup(optarg);
 	    if (!EquivToLoopback) {
@@ -2438,7 +2484,8 @@ main(int argc, char **argv)
 
 	case 'x':
 	    if (scan_body) {
-		free(scan_body);
+                fprintf(stderr, "%s: Cannot use multiple '-x' options\n", argv[0]);
+                exit(EXIT_FAILURE);
 	    }
 	    scan_body = strdup(optarg);
 	    if (!scan_body) {
@@ -2454,7 +2501,8 @@ main(int argc, char **argv)
 	case 'U':
 	    /* User to run as */
 	    if (user) {
-		free(user);
+                fprintf(stderr, "%s: Cannot use multiple '-U' options\n", argv[0]);
+                exit(EXIT_FAILURE);
 	    }
 	    user = strdup(optarg);
 	    if (!user) {
@@ -2464,7 +2512,10 @@ main(int argc, char **argv)
 	    break;
 	case 'o':
 	    /* Use this as our lock file */
-	    if (lockfile != NULL) free(lockfile);
+	    if (lockfile != NULL) {
+                fprintf(stderr, "%s: Cannot use multiple '-o' options\n", argv[0]);
+                exit(EXIT_FAILURE);
+            }
 
 	    lockfile = strdup(optarg);
 	    if (!lockfile) {
@@ -2474,7 +2525,10 @@ main(int argc, char **argv)
 	    break;
 	case 'P':
 	    /* Write our pid to this file */
-	    if (pidfile != NULL) free(pidfile);
+	    if (pidfile != NULL) {
+                fprintf(stderr, "%s: Cannot use multiple '-P' options\n", argv[0]);
+                exit(EXIT_FAILURE);
+            }
 
 	    pidfile = strdup(optarg);
 	    if (!pidfile) {
@@ -2499,6 +2553,10 @@ main(int argc, char **argv)
 	    break;
 	case 'm':
 	    /* Multiplexor */
+            if (MultiplexorSocketName) {
+                fprintf(stderr, "%s: Cannot use multiple '-m' options\n", argv[0]);
+                exit(EXIT_FAILURE);
+            }
 	    MultiplexorSocketName = strdup(optarg);
 	    if (!MultiplexorSocketName) {
 		fprintf(stderr, "%s: Out of memory\n", argv[0]);
@@ -2515,6 +2573,12 @@ main(int argc, char **argv)
 			argv[0], optarg);
 		exit(EXIT_FAILURE);
 	    }
+	    if (sockfile) free(sockfile);
+	    sockfile = strdup(optarg);
+	    if (!sockfile) {
+		fprintf(stderr, "%s: Out of memory\n", argv[0]);
+		exit(EXIT_FAILURE);
+	    }
 	    got_p_option = 1;
 	    /* Remove socket from file system if it's a local socket */
 	    (void) remove_local_socket(optarg);
@@ -2694,7 +2758,7 @@ main(int argc, char **argv)
 
     /* Do the locking */
     if (pidfile || lockfile) {
-	if ( (lockfile_fd = write_and_lock_pidfile(pidfile, lockfile, pidfile_fd)) < 0) {
+	if ( (lockfile_fd = write_and_lock_pidfile(pidfile, &lockfile, pidfile_fd)) < 0) {
 	    /* Signal the waiting parent */
 	    REPORT_FAILURE("Cannot lock lockfile: Is another copy running?");
 	    exit(EXIT_FAILURE);
@@ -2765,6 +2829,9 @@ main(int argc, char **argv)
     if (lockfile) {
 	unlink(lockfile);
     }
+    if (sockfile) {
+	remove_local_socket(sockfile);
+    }
     return rc;
 }
 
